"""
Related items for Django Admin
"""

from django.template import Library, Node, TemplateSyntaxError
from django.template import Variable, VariableDoesNotExist
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse, NoReverseMatch
from django.core.exceptions import ImproperlyConfigured
from django.conf import settings
from collections import defaultdict

_EXCLUDED_MODELS = getattr(settings, "ADMIN_RELATED_EXCLUDES", [])

# Holds a list of "app.model" strings of the models that should be ignored.
EXCLUDED_MODELS = []
for mod in _EXCLUDED_MODELS:
    try:
        a, m = mod.split(".")
        ctype = ContentType.objects.get(app_label__iexact=a, model__iexact=m)
    except ContentType.DoesNotExist:
        raise ImproperlyConfigured(
            "ADMIN_RELATED_EXCLUDES: '%s' is not a valid model" % mod
        )

    EXCLUDED_MODELS.append("%s:%s" % (a.lower(), m.lower()))

# An integer of the number to limit in the queryset.
ITEM_LIMIT = getattr(settings, "ADMIN_RELATED_ITEM_LIMIT", 5)

register = Library()


class RelatedObjectsNode(Node):
    def __init__(self, obj, varname):
        super(RelatedObjectsNode, self).__init__()

        self.obj, self.varname = obj, varname

    def render(self, context):
        """
        Retreive the related objects for a specified instance.
        """
        try:
            obj = Variable(self.obj).resolve(context)
        except VariableDoesNotExist:
            return ""

        rel = defaultdict(list)
        related_models = [x.content_object for x in obj.concepts_conceptitem_items.all()]
        for related in related_models:
            # Get the app and model
            model = related._meta.verbose_name

            rel[model].append(related)

        # Set the return variable to the dictionary.
        context[self.varname] = dict(rel)
        return ""


class AdminUrlNode(Node):
    """
    Returns the admin url to change the object passed as a parameter.

    Optionally will set the value to a variable, otherwise outputs it.
    """

    def __init__(self, obj, varname=None):
        super(AdminUrlNode, self).__init__()

        self.obj = obj
        self.varname = varname

    def render(self, context):
        """
        Either write out the url, or set the variable in the context.
        """
        try:
            # Retrieve the object instance.
            obj = Variable(self.obj).resolve(context)
        except VariableDoesNotExist:
            obj = self.obj

        try:
            urlname = 'admin:%s_%s_change' % (
                obj._meta.app_label, obj._meta.module_name
            )
            url = reverse(urlname, args=(obj.pk,))
        except NoReverseMatch, err:
            if settings.TEMPLATE_DEBUG:
                print "Got an exception resolving an admin url: ", err
            return ''

        if self.varname:
            context[self.varname] = url
        else:
            return url


def get_admin_url(parser, token):
    """
    {% get_admin_url object [as varname ]%}
    """
    argv = token.contents.split()
    argc = len(argv)
    varname = None
    if argc == 0:
        raise TemplateSyntaxError(
            "%s requires an object passed as an argument." % argv[0]
        )
    elif argc == 3:
        raise TemplateSyntaxError(
            "%s requires a variable name passed after 'as'" % argv[0]
        )
    elif argc == 4 and argv[2] != 'as':
        raise TemplateSyntaxError(
            "Usage: {%% %s object [as variablename] %%}" % argv[0]
        )
    elif argc == 4:
        varname = argv[3]

    return AdminUrlNode(argv[1], varname)


def do_related_objects(parser, token):
    """
    Returns the objects related to the object passed. The objects returned are
    objects that have a ForeignKey or ManyToMany relation to this object.

    {% concept_related_objects object as varname %}
    """
    argv = token.contents.split()
    argc = len(argv)

    if argc != 4:
        raise TemplateSyntaxError('Tag %s takes three arguments.' % argv[0])
    if argv[2] != "as":
        raise TemplateSyntaxError(
            'Second argument must be "as" for tag %s' % argv[0]
        )

    return RelatedObjectsNode(argv[1], argv[3])

register.tag("concept_related_objects", do_related_objects)
register.tag("get_admin_url", get_admin_url)



import django
from django.contrib import admin
from django.contrib.contenttypes.generic import GenericTabularInline
from django.db.models import Count
from django import forms

from concepts.models import Concept, ConceptItem
from concepts.settings import WEIGHTS, WIDGET_CSS_PATH

class ConceptAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(ConceptAdminForm, self).__init__(*args, **kwargs)
        instance = kwargs.get('instance', None)
        if instance is not None:
            sub_field = self.fields['substitute']
            sub_field.queryset = sub_field.queryset.exclude(id=instance.id)
    
    class Meta:
        model = Concept

class ConceptItemAdminForm(forms.ModelForm):
    weight = forms.ChoiceField(label="weighting", choices=WEIGHTS, widget=forms.RadioSelect())
    
    class Media:
        css = {
            'all': (WIDGET_CSS_PATH,)
        }
    class Meta:
        model = ConceptItem

class ConceptItemInline(GenericTabularInline):
    form = ConceptItemAdminForm
    model = ConceptItem
    raw_id_fields = ('tag', )
    
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        return super(ConceptItemInline, self).formfield_for_foreignkey(db_field, request, **kwargs)


class ConceptAdmin(admin.ModelAdmin):
    list_display = ["name_with_sub", "last_tagged", "concept_items"]
    form = ConceptAdminForm
    raw_id_fields = ('substitute', )
    prepopulated_fields = {"slug": ("name",)}
    search_fields = ['name']
    fieldsets = (
        (None, {'fields': ('name', 'url',)}),
        ('Moderation', {
            'fields': ('substitute', 'enabled')
        }),
        ('Geolocation', {
            'classes': ('collapse',),
            'fields': ('woeid', ('longitude', 'latitude'), 
                      ('bbox_w', 'bbox_n'), ('bbox_e', 'bbox_s'),)
        }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': ('slug',)
        })
    )
    def concept_items(self, obj):
        return obj.concept_items
    
    def queryset(self, request):
        """
        Return the normal queryset, but add the count of tagged items to it
        """
        if django.VERSION < (1, 2):
            return super(ConceptAdmin, self).queryset(request).annotate(concept_items=Count('conceptitem_items'))
        else:
            return super(ConceptAdmin, self).queryset(request).annotate(concept_items=Count('concepts_conceptitem_items'))

admin.site.register(Concept, ConceptAdmin)



from django import forms
from .settings import WIDGET

class ConceptField(forms.CharField):
    widget = WIDGET
    
    def __init__(self, *args, **kwargs):
        if 'parser' in kwargs:
            if not callable(kwargs['parser']):
                raise ImproperlyConfigured(_("The 'parser' provided must be a callable."))
            
            self.parser = kwargs['parser']
            del(kwargs['parser'])
        else:
            self.parser = parse_tags
        super(ConceptField, self).__init__(*args, **kwargs)
    
    def clean(self, value):
        if isinstance(value, (list, tuple)):
            return list(set(value))
        value = super(ConceptField, self).clean(value)
        try:
            return self.parser(value)
        except ValueError:
            raise forms.ValidationError(_("Please provide a list of concepts."))



# Things to test:

# Substituting replaces all items
# Assigning the tag again actually adds the substitute

# Selecting a disabled tag generates an error
# flag to fail quietly? just doesn't assign it and doesn't generate an error
from django.test import TestCase

from simpleapp.models import SimpleText
from concepts.models import Concept

class SubstitutionTestCase(TestCase):
    def test_substitution(self):
        f = SimpleText.objects.create(name="Good Food")
        f.concepts.add("red", "crunchy", "smooth", "smoothy")
        g = SimpleText.objects.create(name="Bad Food")
        g.concepts.add("red", "crunchy",)
        
        c = Concept.objects.get(name="smoothy")
        r = Concept.objects.get(name="smooth")
        c.substitute = r
        c.save()
        
        # Items were moved to substitute
        self.assertEqual(c.items.count(), 0)
        self.assertEqual(f.concepts.count(), 3)
        
        # Adding it back doesn't do anything because substitute is there
        f.concepts.add("smoothy")
        self.assertEqual(f.concepts.count(), 3)
        
        # Adding it to a new item adds the substitute
        g.concepts.add("smoothy")
        self.assertEqual(g.concepts.count(), 3)
        assert(r in g.concepts.all())
    
    def test_enable(self):
        f = SimpleText.objects.create(name="Good Food")
        f.concepts.add("red", "crunchy", "smooth", "smoothy")
        
        bad_tag = Concept.objects.get(name="smoothy")
        bad_tag.enabled=False
        bad_tag.save()
        
        self.assertEqual(f.concepts.count(), 3)
        
        f.concepts.add("smoothy")
        self.assertEqual(f.concepts.count(), 3)
        print f.concepts.all()


from django import forms
from django.db.models.query import QuerySet
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django.utils import simplejson

from taggit.utils import edit_string_for_tags

class TextExtWidget(forms.TextInput):
    input_type = 'text'

    def render(self, name, value, attrs=None):
        list_view = reverse('concepts-autocomplete-list')
        if value is not None:
            if isinstance(value, QuerySet):
                value = [o.tag.name for o in value.select_related("tag")]
            elif isinstance(value, basestring):
                value = [value]
        else:
            value = []
        html = super(TextExtWidget, self).render(name, '', attrs)
        js = render_to_string('concepts/textext.html', {
            "element_id": attrs['id'], 
            "autocomplete_url": list_view,
            "items": simplejson.dumps(list(value))
        })
        return mark_safe("\n".join([html, js]))
    
    def value_from_datadict(self, data, files, name):
        tags = simplejson.loads(data[name])
        return tags
    
    class Media:
        css = {'all':('concepts/textext.css',)}
        js = ('concepts/jquery-textext.js',)



from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.views.generic.list_detail import object_list
from django.utils import simplejson

from concepts.models import ConceptItem, Concept


def tagged_object_list(request, slug, queryset, **kwargs):
    if callable(queryset):
        queryset = queryset()
    tag = get_object_or_404(Concept, slug=slug)
    qs = queryset.filter(pk__in=ConceptItem.objects.filter(
        tag=tag, content_type=ContentType.objects.get_for_model(queryset.model)
    ).values_list("object_id", flat=True))
    if "extra_context" not in kwargs:
        kwargs["extra_context"] = {}
    kwargs["extra_context"]["tag"] = tag
    return object_list(request, qs, **kwargs)


def list_tags(request):
    term = request.GET.get('term', '')
    if not term:
        term = request.GET.get('q', '')
    if term:
        tag_items = Concept.objects.filter(name__istartswith=term).values_list(
            'name', 'substitute__name', 'enabled')
        tags = []
        for item in tag_items:
            if item[1]:
                tags.append("%s|%s" % (item[0], item[1]))
            elif not item[2]:
                tags.append("%s|-" % item[0])
            else:
                tags.append(item[0])
    else:
        tags = []

    return HttpResponse(simplejson.dumps(list(tags)), mimetype='application/javascript')



from django.conf import settings

DEFAULT_SETTINGS = {
    'JS_BASE_URL': settings.STATIC_URL+"concepts/",
    'PARSER': 'concepts.parsers.default',
    'WIDGET': 'concepts.widgets.TextExtWidget',
    'WEIGHTS': ((0, 'Hide'), (10, 'Low'), (20, 'Medium'), (30, 'High')),
    'WIDGET_CSS_PATH': 'concepts/horizontalradio.css',
}

USER_SETTINGS = DEFAULT_SETTINGS.copy()
USER_SETTINGS.update(getattr(settings, 'CONCEPTS_SETTINGS', {}))

if callable(USER_SETTINGS['PARSER']):
    pass
elif isinstance(USER_SETTINGS['PARSER'], basestring):
    from django.utils.importlib import import_module
    bits = USER_SETTINGS['PARSER'].split(".")
    module = import_module(".".join(bits[:-1]))
    USER_SETTINGS['PARSER'] = getattr(module, bits[-1])
else:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured("PARSER must be a callable or a string.")

if isinstance(USER_SETTINGS['WIDGET'], basestring):
    from django.utils.importlib import import_module
    bits = USER_SETTINGS['WIDGET'].split(".")
    module = import_module(".".join(bits[:-1]))
    USER_SETTINGS['WIDGET'] = getattr(module, bits[-1])
else:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured("WIDGET must be a string.")

globals().update(USER_SETTINGS)


from django import template
from django.core.exceptions import FieldError

from concepts.models import ConceptItem, Concept
from django.contrib.contenttypes.models import ContentType

register = template.Library()


class ConceptsNode(template.Node):
    """
    Sets a context variable to the concepts linked to the object
    """
    def __init__(self, obj, variable_name):
        self.obj = template.Variable(obj)
        self.variable_name = variable_name

    def render(self, context):
        obj = self.obj.resolve(context)
        ctype = ContentType.objects.get_for_model(obj)
        links = ConceptItem.objects.filter(object_id=obj.pk, content_type=ctype)
        context[self.variable_name] = links
        return ''

@register.tag
def get_concepts_for_object(parser, token):
    """
    Get the concepts linked to the object instance passed

    {% get_concepts_for_object object as var %}
    """
    bits = token.split_contents()

    if len(bits) != 4:
        raise template.TemplateSyntaxError("The proper usage is: {%% get_concepts_for_object object as var %}")

    return ConceptsNode(bits[1], bits[3])

@register.filter(name='li_class')
def li_class(weight):
    return 5 - (weight / 5)



import datetime

import django
from django.contrib.contenttypes.models import ContentType
from django.db import connection, models
from django.utils.translation import ugettext_lazy as _

from taggit.models import TagBase, GenericTaggedItemBase, ItemBase

qn = connection.ops.quote_name


def get_queryset_and_model(queryset_or_model):
    """
    Given a ``QuerySet`` or a ``Model``, returns a two-tuple of
    (queryset, model).

    If a ``Model`` is given, the ``QuerySet`` returned will be created
    using its default manager.
    """
    try:
        return queryset_or_model, queryset_or_model.model
    except AttributeError:
        return queryset_or_model._default_manager.all(), queryset_or_model


class ConceptModelManager(models.Manager):
    def get_intersection_by_model(self, queryset_or_model, tags):
        """
        Create a ``QuerySet`` containing instances of the specified
        model associated with *all* of the given list of tags.
        """
        tag_count = len(tags)
        queryset, model = get_queryset_and_model(queryset_or_model)

        if not tag_count:
            return model._default_manager.none()

        model_table = qn(model._meta.db_table)
        # This query selects the ids of all objects which have all the
        # given tags.
        query = """
        SELECT %(model_pk)s
        FROM %(model)s, %(tagged_item)s
        WHERE %(tagged_item)s.content_type_id = %(content_type_id)s
          AND %(tagged_item)s.tag_id IN (%(tag_id_placeholders)s)
          AND %(model_pk)s = %(tagged_item)s.object_id
        GROUP BY %(model_pk)s
        HAVING COUNT(%(model_pk)s) = %(tag_count)s""" % {
            'model_pk': '%s.%s' % (model_table, qn(model._meta.pk.column)),
            'model': model_table,
            'tagged_item': qn(self.model._meta.db_table),
            'content_type_id': ContentType.objects.get_for_model(model).pk,
            'tag_id_placeholders': ','.join(['%s'] * tag_count),
            'tag_count': tag_count,
        }

        cursor = connection.cursor()
        cursor.execute(query, [tag.pk for tag in tags])
        object_ids = [row[0] for row in cursor.fetchall()]
        if len(object_ids) > 0:
            return queryset.filter(pk__in=object_ids)
        else:
            return model._default_manager.none()

    def get_union_by_model(self, queryset_or_model, tags):
        """
        Create a ``QuerySet`` containing instances of the specified
        model associated with *any* of the given list of tags.
        """
        tag_count = len(tags)
        queryset, model = get_queryset_and_model(queryset_or_model)

        if not tag_count:
            return model._default_manager.none()

        model_table = qn(model._meta.db_table)
        # This query selects the ids of all objects which have any of
        # the given tags.
        query = """
        SELECT %(model_pk)s
        FROM %(model)s, %(tagged_item)s
        WHERE %(tagged_item)s.content_type_id = %(content_type_id)s
          AND %(tagged_item)s.tag_id IN (%(tag_id_placeholders)s)
          AND %(model_pk)s = %(tagged_item)s.object_id
        GROUP BY %(model_pk)s""" % {
            'model_pk': '%s.%s' % (model_table, qn(model._meta.pk.column)),
            'model': model_table,
            'tagged_item': qn(self.model._meta.db_table),
            'content_type_id': ContentType.objects.get_for_model(model).pk,
            'tag_id_placeholders': ','.join(['%s'] * tag_count),
        }

        cursor = connection.cursor()
        cursor.execute(query, [tag.pk for tag in tags])
        object_ids = [row[0] for row in cursor.fetchall()]
        if len(object_ids) > 0:
            return queryset.filter(pk__in=object_ids)
        else:
            return model._default_manager.none()

    def _get_usage(self, model, counts=False, min_count=None, extra_joins=None, extra_criteria=None, params=None):
        """
        Perform the custom SQL query for ``usage_for_model`` and
        ``usage_for_queryset``.
        """
        if min_count is not None:
            counts = True

        model_table = qn(model._meta.db_table)
        model_pk = '%s.%s' % (model_table, qn(model._meta.pk.column))
        query = """
        SELECT DISTINCT %(tag)s.id, %(tag)s.name%(count_sql)s
        FROM
            %(tag)s
            INNER JOIN %(tagged_item)s
                ON %(tag)s.id = %(tagged_item)s.tag_id
            INNER JOIN %(model)s
                ON %(tagged_item)s.object_id = %(model_pk)s
            %%s
        WHERE %(tagged_item)s.content_type_id = %(content_type_id)s
            %%s
        GROUP BY %(tag)s.id, %(tag)s.name
        %%s
        ORDER BY %(tag)s.name ASC""" % {
            'tag': qn(self.model._meta.db_table),
            'count_sql': counts and (', COUNT(%s)' % model_pk) or '',
            'tagged_item': qn(ConceptItem._meta.db_table),
            'model': model_table,
            'model_pk': model_pk,
            'content_type_id': ContentType.objects.get_for_model(model).pk,
        }

        min_count_sql = ''
        if min_count is not None:
            min_count_sql = 'HAVING COUNT(%s) >= %%s' % model_pk
            params.append(min_count)

        cursor = connection.cursor()
        cursor.execute(query % (extra_joins, extra_criteria, min_count_sql), params)
        tags = []
        for row in cursor.fetchall():
            t = self.model(*row[:2])
            if counts:
                t.count = row[2]
            tags.append(t)
        return tags

    def usage_for_model(self, model, counts=False, min_count=None, filters=None):
        """
        Obtain a list of tags associated with instances of the given
        Model class.

        If ``counts`` is True, a ``count`` attribute will be added to
        each tag, indicating how many times it has been used against
        the Model class in question.

        If ``min_count`` is given, only tags which have a ``count``
        greater than or equal to ``min_count`` will be returned.
        Passing a value for ``min_count`` implies ``counts=True``.

        To limit the tags (and counts, if specified) returned to those
        used by a subset of the Model's instances, pass a dictionary
        of field lookups to be applied to the given Model as the
        ``filters`` argument.
        """
        if filters is None:
            filters = {}

        queryset = model._default_manager.filter()
        for f in filters.items():
            queryset.query.add_filter(f)
        usage = self.usage_for_queryset(queryset, counts, min_count)

        return usage

    def usage_for_queryset(self, queryset, counts=False, min_count=None):
        """
        Obtain a list of tags associated with instances of a model
        contained in the given queryset.

        If ``counts`` is True, a ``count`` attribute will be added to
        each tag, indicating how many times it has been used against
        the Model class in question.

        If ``min_count`` is given, only tags which have a ``count``
        greater than or equal to ``min_count`` will be returned.
        Passing a value for ``min_count`` implies ``counts=True``.
        """

        if getattr(queryset.query, 'get_compiler', None):
            # Django 1.2+
            compiler = queryset.query.get_compiler(using='default')
            extra_joins = ' '.join(compiler.get_from_clause()[0][1:])
            where, params = queryset.query.where.as_sql(
                compiler.quote_name_unless_alias, compiler.connection
            )
        else:
            # Django pre-1.2
            extra_joins = ' '.join(queryset.query.get_from_clause()[0][1:])
            where, params = queryset.query.where.as_sql()

        if where:
            extra_criteria = 'AND %s' % where
        else:
            extra_criteria = ''
        return self._get_usage(queryset.model, counts, min_count, extra_joins, extra_criteria, params)


class Concept(TagBase):
    """An idea or other freeform categorization of something"""

    created = models.DateTimeField(_('created'),
        auto_now_add=True,
        editable=False)
    last_tagged = models.DateTimeField(_('last time tagged'),
        blank=True, null=True, editable=False, db_index=True)
    substitute = models.ForeignKey('Concept',
        blank=True, null=True, verbose_name=_('substitute'),
        help_text=_("""Tag to use instead of this one. Moves current
            associations to the substitute tag and new association attempts
            are automatically swapped."""))
    enabled = models.BooleanField(_("Enabled"), default=True,
        help_text=_("""If unchecked, it will remove current associations and
            will not allow new associations."""))
    url = models.CharField(blank=True, max_length=255,
        help_text=_("A URL for more information regarding this concept."))
    woeid = models.IntegerField(_('where on earth id'), blank=True, null=True)
    latitude = models.DecimalField(_('latitude'),
        max_digits=11, decimal_places=6, blank=True, null=True)
    longitude = models.DecimalField(_('longitude'),
        max_digits=11, decimal_places=6, blank=True, null=True)
    bbox_n = models.DecimalField(_('bounding box north'),
        max_digits=11, decimal_places=6, blank=True, null=True)
    bbox_s = models.DecimalField(_('bounding box south'),
        max_digits=11, decimal_places=6, blank=True, null=True)
    bbox_e = models.DecimalField(_('bounding box east'),
        max_digits=11, decimal_places=6, blank=True, null=True)
    bbox_w = models.DecimalField(_('bounding box west'),
        max_digits=11, decimal_places=6, blank=True, null=True)

    objects = ConceptModelManager()

    @property
    def items(self):
        if django.VERSION < (1, 2):
            return self.conceptitem_items
        else:
            return self.concepts_conceptitem_items

    def name_with_sub(self):
        """
        Render the name, or name with indication what its substitute is
        """
        if self.substitute:
            return "%s &rarr; %s" % (self.name, self.substitute)
        elif not self.enabled:
            return '<span style="text-decoration: line-through">%s</span>' % self.name
        else:
            return self.name
    name_with_sub.short_description = _("Name")
    name_with_sub.admin_order_field = "name"
    name_with_sub.allow_tags = True

    def save(self, *args, **kwargs):
        if not self.id:
            self.created = datetime.datetime.today()
        super(Concept, self).save(*args, **kwargs)
        if self.substitute:
            items = self.items.all()
            items.update(tag=self.substitute)
        if not self.enabled:
            self.items.all().delete()

    class Meta:
        verbose_name = _("Concept")
        verbose_name_plural = _("Concepts")
        ordering = ("name",)


class ConceptItemBase(ItemBase):
    if django.VERSION < (1, 2):
        tag = models.ForeignKey(Concept, related_name="%(class)s_items")
    else:
        tag = models.ForeignKey(Concept, related_name="%(app_label)s_%(class)s_items")

    class Meta:
        abstract = True

    @classmethod
    def tags_for(cls, model, instance=None):
        if instance is not None:
            return cls.tag_model().objects.filter(**{
                '%s__content_object' % cls.tag_relname(): instance
            })
        return cls.tag_model().objects.filter(**{
            '%s__content_object__isnull' % cls.tag_relname(): False
        }).distinct()


class ConceptItem(GenericTaggedItemBase, ConceptItemBase):
    added = models.DateTimeField(auto_now_add=True, db_index=True)
    weight = models.IntegerField(blank=True, null=True)

    def save(self, *args, **kwargs):
        """
        Add the date added and last_tagged to Tag
        """
        if not self.added:
            self.added = datetime.datetime.now()

        super(ConceptItem, self).save(*args, **kwargs)
        self.tag.last_tagged = self.added
        self.tag.save()

    class Meta:
        verbose_name = _("Concept Item")
        verbose_name_plural = _("Concept Items")
        ordering = ('id',)


# The association between concepts and a related item must be
# deleted when the item is deleted
def delete_listener(sender, instance, **kwargs):
    ctype = ContentType.objects.get_for_model(sender)
    ConceptItem.objects.filter(content_type=ctype,
                               object_id=instance.id).delete()



from taggit.utils import parse_tags

def default(value):
    if isinstance(value, (list, tuple)):
        return list(set(value))
    else:
        return parse_tags(value)

def json_parser(value):
    from django.utils import simplejson
    
    if value:
        if isinstance(value, (list, tuple)):
            return list(set(value))
        else:
            return simplejson.loads(value)
    return value



from django.conf.urls.defaults import *
from django.conf import settings

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    (r'^concepts/', include('concepts.urls')),
    (r'^admin/', include(admin.site.urls)),
)

urlpatterns = urlpatterns + patterns('',
    (r'^static/(?P<path>.*)$', 'django.views.static.serve',
        {'document_root': settings.STATIC_ROOT}),
    ) if settings.DEBUG else urlpatterson




from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django.utils.text import capfirst

from taggit.managers import TaggableManager, _get_subclasses

from .models import ConceptItem
from .forms import ConceptField
from .settings import PARSER


class ConceptManager(TaggableManager):
    def __init__(self, verbose_name=_("Concepts"),
        help_text=_("Press return to complete each tag."),
        through=ConceptItem, blank=False):
        super(ConceptManager, self).__init__(verbose_name, help_text, through, blank)

    def extra_filters(self, pieces, pos, negate):
        if negate or not self.use_gfk:
            return []
        prefix = "__".join(["concept_items"] + pieces[:pos - 2])
        cts = map(ContentType.objects.get_for_model, _get_subclasses(self.model))
        if len(cts) == 1:
            return [("%s__content_type" % prefix, cts[0])]
        return [("%s__content_type__in" % prefix, cts)]

    def formfield(self, form_class=ConceptField, **kwargs):
        defaults = {
            "label": capfirst(self.verbose_name),
            "help_text": self.help_text,
            "required": not self.blank,
            "parser": PARSER,
        }
        defaults.update(kwargs)
        return form_class(**defaults)
